package cn.com.qmth.stmms.biz.exam.service.impl;

import cn.com.qmth.stmms.biz.common.BaseQueryService;
import cn.com.qmth.stmms.biz.exam.dao.MarkerDao;
import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.query.MarkerSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.biz.mark.dao.MarkLibraryDao;
import cn.com.qmth.stmms.common.enums.LibraryStatus;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.LinkedList;
import java.util.List;

@Service("markerService")
public class MarkerServiceImpl extends BaseQueryService<Marker> implements MarkerService {

    @Autowired
    private MarkerDao markerDao;

    @Autowired
    private MarkLibraryDao libraryDao;

    @Transactional
    @Override
    public Marker save(Marker marker) {
        // Marker previous = findByLoginName(marker.getLoginName());
        // if (previous == null || (marker.getId() != null &&
        // previous.getId().equals(marker.getId()))) {
        return markerDao.save(marker);
        // } else {
        // return null;
        // }
    }

    @Transactional
    @Override
    public void updateMarkSetting(Integer id, String setting) {
        markerDao.updateMarkSettingById(id, setting);
    }

    @Override
    public Marker findById(Integer id) {
        return markerDao.findOne(id);
    }

    @Override
    public List<Marker> findByExamAndSubjectAndGroup(int examId, String subjectCode, int number) {
        return markerDao.findByExamIdAndSubjectCodeAndGroupNumber(examId, subjectCode, number, null);
    }

    @Override
    public long countByExam(int examId) {
        return markerDao.countByExamId(examId);
    }

    @Override
    public long countByExamAndSubjectAndGroup(int examId, String subjectCode, int number) {
        return markerDao.countByExamIdAndSubjectCodeAndGroupNumber(examId, subjectCode, number);
    }

    @Override
    public List<Marker> getMarkCount(int examId) {
        List<Marker> list = new LinkedList<Marker>();
        List<Object[]> result = libraryDao.countMarkerAndStatus(examId, LibraryStatus.MARKED, LibraryStatus.INSPECTED);
        if (result != null) {
            for (Object[] array : result) {
                try {
                    Marker marker = findById((int) array[0]);
                    marker.setMarkedCount((long) array[1]);
                    list.add(marker);
                } catch (Exception e) {
                }
            }
        }
        return list;
    }

    @Override
    public List<Marker> getMarkCount(int examId, String subjectCode) {
        List<Marker> list = new LinkedList<Marker>();
        List<Object[]> result = libraryDao
                .countMarkerAndStatus(examId, subjectCode, LibraryStatus.MARKED, LibraryStatus.INSPECTED);
        if (result != null) {
            for (Object[] array : result) {
                try {
                    Marker marker = findById((int) array[0]);
                    marker.setMarkedCount((long) array[1]);
                    list.add(marker);
                } catch (Exception e) {
                }
            }
        }
        return list;
    }

    @Override
    public MarkerSearchQuery findByQuery(final MarkerSearchQuery query) {
        checkQuery(query);
        Page<Marker> result = markerDao.findAll(buildSpecification(query), query);
        fillResult(result, query);
        return query;
    }

    private Specification<Marker> buildSpecification(final MarkerSearchQuery query) {
        return new Specification<Marker>() {

            @Override
            public Predicate toPredicate(Root<Marker> root, CriteriaQuery<?> cQuery, CriteriaBuilder cb) {
                List<Predicate> predicates = new LinkedList<Predicate>();
                if (query.getId() != null) {
                    predicates.add(cb.equal(root.get("id"), query.getId()));
                }
                if (query.getExamId() != null) {
                    predicates.add(cb.equal(root.get("examId"), query.getExamId()));
                }
                if (StringUtils.isNotBlank(query.getSubjectCode())) {
                    predicates.add(cb.equal(root.get("subjectCode"), query.getSubjectCode()));
                }
                if (query.getGroupNumber() != null) {
                    predicates.add(cb.equal(root.get("groupNumber"), query.getGroupNumber()));
                }
                if (query.getUserId() != null) {
                    predicates.add(cb.equal(root.get("userId"), query.getUserId()));
                }
                return predicates.isEmpty() ?
                        cb.conjunction() :
                        cb.and(predicates.toArray(new Predicate[predicates.size()]));
            }
        };
    }

    @Override
    public int batchSave(List<Marker> list) {
        markerDao.save(list);
        return list.size();
    }

    @Override
    public List<Marker> findMode(String commo) {
        return markerDao.findByMode(commo);
    }

}
